@@ -75,7 +75,11 @@ References:
 import re
 import urlparse
 import sys
-from string import Template
+try:
+    from string import Template
+except ImportError:
+    from webob.util.stringtemplate import Template
+import types
 from webob import Response, Request, html_escape
 
 tag_re = re.compile(r'<.*?>', re.S)
@@ -100,7 +104,25 @@ def strip_tags(value):
     value = tag_re.sub('', value)
     return value
 
-class HTTPException(Response, Exception):
+class HTTPException(Exception):
+    """
+    Exception used on pre-Python-2.5, where new-style classes cannot be used as
+    an exception.
+    """
+
+    def __init__(self, message, wsgi_response):
+        Exception.__init__(self, message)
+        self.wsgi_response = wsgi_response
+
+    def __call__(self, environ, start_response):
+        return self.wsgi_response(environ, start_response)
+
+    def exception(self):
+        return self
+    
+    exception = property(exception)
+
+class WSGIHTTPException(Response, HTTPException):
 
     ## You should set in subclasses:
     # code = 200
@@ -138,8 +160,9 @@ ${body}''')
     def __init__(self, detail=None, headers=None, comment=None,
                  body_template=None):
         Response.__init__(self,
-                          status = '%s %s' % (self.code, self.title),
+                          status='%s %s' % (self.code, self.title),
                           content_type='text/html')
+        Exception.__init__(self, detail)
         if headers:
             self.headers.update(headers)
         self.detail = detail
@@ -159,7 +182,7 @@ ${body}''')
         else:
             args['html_comment'] = ''
         body_tmpl = self.body_template_obj
-        if HTTPException.body_template_obj is not self.body_template_obj:
+        if WSGIHTTPException.body_template_obj is not self.body_template_obj:
             # Custom template; add headers to args
             for k, v in environ.items():
                 args[k] = escape(v)
@@ -205,7 +228,20 @@ ${body}''')
             return self.generate_response(environ, start_response)
         return Response.__call__(self, environ, start_response)
 
-class HTTPError(HTTPException):
+    def wsgi_response(self):
+        return self
+    
+    wsgi_response = property(wsgi_response)
+
+    def exception(self):
+        if sys.version_info >= (2, 5):
+            return self
+        else:
+            return HTTPException(self.detail, self)
+
+    exception = property(exception)
+
+class HTTPError(WSGIHTTPException):
     """
     base class for status codes in the 400's and 500's
 
@@ -214,7 +250,7 @@ class HTTPError(HTTPException):
     typically results in the 400's and 500's.
     """
 
-class HTTPRedirection(HTTPException):
+class HTTPRedirection(WSGIHTTPException):
     """
     base class for 300's status code (redirections)
 
@@ -224,7 +260,7 @@ class HTTPRedirection(HTTPException):
     condition.
     """
 
-class HTTPOk(HTTPException):
+class HTTPOk(WSGIHTTPException):
     """
     Base class for the 200's status code (successful responses)
     """
@@ -589,17 +625,18 @@ else:
     for name in dir(httpexceptions):
         obj = globals().get(name)
         if (obj and isinstance(obj, type) and issubclass(obj, HTTPException)
-            and obj is not HTTPException):
+            and obj is not HTTPException
+            and obj is not WSGIHTTPException):
             obj.__bases__ = obj.__bases__ + (getattr(httpexceptions, name),)
     del name, obj, httpexceptions
 
 __all__ = ['HTTPExceptionMiddleware', 'status_map']
 status_map={}
 for name, value in globals().items():
-    if (isinstance(value, type) and issubclass(value, HTTPException)
+    if (isinstance(value, (type, types.ClassType)) and issubclass(value, HTTPException)
         and not name.startswith('_')):
         __all__.append(name)
-        if value.code:
+        if getattr(value, 'code', None):
             status_map[value.code]=value
 del name, value
 
